Attribute VB_Name = "Module1"
Option Explicit

'***********************************************************
' Command Codes for the FT847
' Check your manual for other models and adapt accordingly
'***********************************************************

' Set CAT Mode On/Off
Public Const Set_CAT_On = &H0              ' Set CAT mode On
Public Const Set_CAT_Off = &H80            ' Set CAT mode Off

' Set PTT On/Off
Public Const Set_PTT_On = &H8              ' Set PTT On
Public Const Set_PTT_Off = &H88            ' Set PTT Off

' Set Satellite On/Off
Public Const Set_Satellite_On = &H4E       ' Set Satellite mode On
Public Const Set_Satellite_Off = &H8E      ' Set Satellite mode Off

' Set Frequency
Public Const Set_Freq_Main = &H1           ' Set Main VFO frequency
Public Const Set_Freq_Sat_RX = &H11        ' Set SAT RX frequency
Public Const Set_Freq_Sat_TX = &H21        ' Set SAT TX frequency

' Set Operating Mode
Public Const Set_Mode_LSB = &H0            ' Set Mode LSB
Public Const Set_Mode_USB = &H1            ' Set Mode USB
Public Const Set_Mode_CW = &H2             ' Set Mode CW
Public Const Set_Mode_CWR = &H3            ' Set Mode CW-R
Public Const Set_Mode_AM = &H4             ' Set Mode AM
Public Const Set_Mode_FM = &H8             ' Set Mode FM
Public Const Set_Mode_CWN = &H82           ' Set Mode CW(N)
Public Const Set_Mode_CWRN = &H83          ' Set Mode CW-R(N)
Public Const Set_Mode_AMN = &H84           ' Set Mode AM(N)
Public Const Set_Mode_FMN = &H88           ' Set Mode FM(N)
Public Const Set_Mode_Main = &H7           ' Set Mode on Main VFO
Public Const Set_Mode_Sat_RX = &H17        ' Set Mode on SAT RX
Public Const Set_Mode_Sat_TX = &H27        ' Set Mode on SAT TX

' Read Frequency & Mode
Public Const Read_Freq_Mode_Main = &H3     ' Read Frequency & mode on main VFO
Public Const Read_Freq_Mode_Sat_RX = &H13  ' Read Frequency & Mode on Sat RX VFO
Public Const Read_Freq_Mode_Sat_TX = &H23  ' Read Frequency & Mode on Sat TX VFO

' Set CTCSS/DCS mode
Public Const Set_DCS_On = &HA              ' Set DCS On
Public Const Set_CTSS_ENC_DEC_On = &H2A    ' Set CTCSS ENC/DEC On
Public Const Set_CTCSS_ENC_On = &H4A       ' Set CTCSS ENC On
Public Const Set_CTCSS_DCS_On = &H8A       ' Set CTCSS/DCS On
Public Const Set_CTCSS_Main = &HA          ' Set Main VFO
Public Const Set_CTCSS_SAT_RX = &H1A       ' Set SAT RX VFO
Public Const Set_CTCSS_SAR_TX = &H2A       ' Set SAT TX VFO

' CTCSS Frequency
Public Const Set_CTCSS_Frequency_Main = &HB ' Set Main VFO
Public Const Set_CTCSS_Frequency_Sat_RX = &H1B ' Set SAT RX VFO
Public Const Set_CTCSS_Frequency_Sat_TX = &H2B ' Set SAT TX VFO

' DCS Code
Public Const Set_DCS_Main = &HC            ' Set Main VFO
Public Const Set_DCS_Sat_RX = &H1C         ' Set SAT RX VFO
Public Const Set_DCS_Sat_TX = &H2C         ' Set SAT TX VFO

' Repeater Shift
Public Const Set_Repeater_Shift = &H9          ' Set  repeater shift opcode
Public Const Set_Repeater_Shift_Minus = &H9    ' Set repeater shift minus
Public Const Set_Repeater_Shift_Plus = &H49    ' Set repeater shift plus
Public Const Set_Repeater_Shift_Simplex = &H89 ' Set repeater shift simplex

' Repeater Offset
Public Const Set_Repeater_Offset = &HF9    ' Set  repeater shift opcode

' Receiver Status
Public Const Read_Receiver_Status = &HE7   ' Read Receiver Status

' Transmit Status
Public Const Read_Transmit_Status = &HF7   ' Read Transmit Status

' end of constants for FT847 command codes
'***********************************************************

Public Const Memories = "Memories.txt"  ' name of memories file

Public DecimalsOn As Boolean            ' true=show decimals on display
Public ShowFrequency As String          ' displayed frequency
Public Frequency As String              ' decoded frequency with leading zeros
Public Ret As Long                      ' general return code variable
Public I As Integer                     ' loop counter
Public Cmd(4) As Byte                   ' 5 byte command string for FT847
Public FreqString As String             ' work area for frequency
Public FreqLong As Long                 ' frequency in long format
Public FreqLongTemp As Long             ' temporary frequency in long format
Public FreqArray(7) As Integer          ' work array for frequency
Public RxData() As Byte                 ' returned data from FT847
Public StopAction As Boolean            ' true=stop action
Public Mode As String                   ' work area for mode
Public CommSpeed As String              ' speed setting for Comm port
Public CommPort As Integer              ' Comm port number
Public FreqLowChange As Boolean         ' flags a change of value to sldLow
Public FreqMidChange As Boolean         ' flags a change of value to sldMid
Public FreqHighChange As Boolean        ' flags a change of value to sldHigh
Public FreqLowBeingChanged As Boolean   ' flags sldLow being changed
Public FreqMidBeingChanged As Boolean   ' flags sldMid being changed
Public FreqHighBeingChanged As Boolean  ' flags sldHigh being changed
Public FreqMhzPlus As Boolean           ' flags +1 Mhz
Public FreqMhzMinus As Boolean          ' flags -1 Mhz
Public FrequencyError As Boolean        ' flags a frequency out of band
Public MemName() As String              ' memory element name
Public MemFreq() As String              ' memory frequency in Khz
Public MemMode() As String              ' memory mode
Public MemNumberOf As Integer           ' number of memories
Public SettingsRequested As Boolean     ' flag settings form requested

Public Sub InitialiseVariables()
'***********************************************************
' initialise all variables that need it
'***********************************************************
Dim Work1 As String                     ' temp work area 1
Dim Work2 As String                     ' temp work area 2
Dim Work3 As String                     ' temp work area 3

    StopAction = False                  ' do not halt polling
    DecimalsOn = True                   ' show decimals in display
    Form1.Timer1.Interval = 100         ' msec timer interval
    FreqLowChange = False               ' init no change to sldLow value
    FreqMidChange = False               ' init no change to sldMid value
    FreqLowBeingChanged = False         ' init sldLow being changed
    FreqMidBeingChanged = False         ' init sldMid being changed
    FreqMhzPlus = False                 ' init not + 1 Mhz
    FreqMhzMinus = False                ' init not - 1 Mhz
    FrequencyError = False              ' init no frequency error
    SettingsRequested = False           ' init settings not requested
    
    ' get the settings for Commport and Commspeed from registry.
    ' If nothing there then defaults to "2" and "57600,N,8,1"
    CommPort = Val(GetSetting("DriveIt", "Settings", "Commport", "1"))
    CommSpeed = GetSetting("DriveIt", "Settings", "CommSpeed", "4800,N,8,2")
    
    ' load memories from file
    ' see if exists
    If UCase$(Dir(Memories)) <> "" Then ' if got a file
        
        ' count number of records in file
        Open Memories For Input As #1   ' open memories file
        I = 0                           ' initialise index to array
        Do While Not EOF(1)             ' while not end of file...
            Input #1, Work1, Work2, Work3
            I = I + 1                   ' increment array index
        Loop                            ' get next record
        Close #1                        ' end of file or array full - close file
        MemNumberOf = I - 1             ' save number of memories in use
        
        ' allocate arrays to hold all records in file
        ReDim MemName(MemNumberOf)
        ReDim MemFreq(MemNumberOf)
        ReDim MemMode(MemNumberOf)
        
        Open Memories For Input As #1   ' open memories file
        I = 0                           ' initialise index to array
        Do While Not EOF(1)             ' while not end of file...
            Input #1, MemName(I), MemFreq(I), MemMode(I)
            Form1.cboMemories.AddItem MemName(I) ' add to combo box
            I = I + 1                   ' increment array index
            If I > UBound(MemName) Then Exit Do ' if end of array space
        Loop                            ' get next record
        Close #1                        ' end of file or array full - close file
        MemNumberOf = I - 1             ' save number of memories in use
    End If

End Sub

Public Sub SetCommPort(Port As Integer, Speed As String)
'***********************************************************
' set up com2 for talking to the radio
'***********************************************************
    
    ' close comm port if already open
    If Form1.MSComm1.PortOpen = True Then
        Form1.MSComm1.PortOpen = False
    End If
    
    ' Communications port settings.
    Form1.MSComm1.CommPort = Port             ' port number
    Form1.MSComm1.Settings = Speed            ' match the speed on your transceiver
                                              ' by using Menu 37, CAT function
    Form1.MSComm1.InputMode = comInputModeBinary
    Form1.MSComm1.InputLen = 0
    
    ' Open the communications port.
    On Error Resume Next
    Form1.MSComm1.PortOpen = True
    If Err Then
       MsgBox "COM Port not available. Change the CommPort to another port."
       Exit Sub
    End If
    
    ' Clear the thresholds
    Form1.MSComm1.RThreshold = 0
    Form1.MSComm1.SThreshold = 0
    
End Sub

Public Sub MakeFreqKhz(FreqWanted As Long)
'***********************************************************
' Convert a 'long' frequency in Khz to the Cmd() byte array to send to
' the transceiver.
' Usage
'       MakeFreqKhz <frequency in Khz>
'***********************************************************

    ' validate proposed frequency
    If FrequencyValidate(FreqWanted * 100) = False Then Exit Sub

    ' convert long value to a string of 8 chars with leading zeros
    FreqString = Format(FreqWanted * 100, "00000000")
    
    ' turn into array of numbers wanted by subtracting 48
    ' from the ASCII value of each byte
    For I = 0 To 7
        FreqArray(I) = Asc(Mid(FreqString, I + 1, 1)) - 48
    Next

    ' now add each pair of numbers for each byte of Cmd()
    For I = 0 To 3
        Cmd(I) = CByte((FreqArray(I * 2) * 16) + FreqArray(I * 2 + 1))
    Next

End Sub

Public Sub MakeFreqHz(FreqWanted As Long)
'***********************************************************
' Convert a 'long' frequency in Hz to the Cmd() byte array to send to
' the transceiver.
' Usage
'       MakeFreqHz <frequency in 10Hz>
'***********************************************************

    ' validate proposed frequency
    If FrequencyValidate(FreqWanted) = False Then Exit Sub
    
    ' convert long value to a string of 8 chars with leading zeros
    FreqString = Format(FreqWanted, "00000000")
    
    ' turn into array of numbers wanted by subtracting 48
    ' from the ASCII value of each byte
    For I = 0 To 7
        FreqArray(I) = Asc(Mid(FreqString, I + 1, 1)) - 48
    Next

    ' now add each pair of numbers for each byte of Cmd()
    For I = 0 To 3
        Cmd(I) = CByte((FreqArray(I * 2) * 16) + FreqArray(I * 2 + 1))
    Next

End Sub

Public Function FrequencyValidate(FrequencyToCheck As Long)
'***********************************************************
' validate a frequency in 10Hz is within transceiver range
'***********************************************************

    Select Case FrequencyToCheck
        Case 10000 To 7600000
        Case 10800000 To 17400000
        Case 42000000 To 51200000
        Case Else
            FrequencyValidate = False           ' return False
            FrequencyError = True               ' flag bad frequency
            Exit Function
    End Select
    
    FrequencyValidate = True                    ' return True
    FrequencyError = False                      ' flag not bad frequenncy

End Function

Public Sub ReadSmeter()
'***********************************************************
' Read the S meter procedure
'***********************************************************

Dim Reading As Integer
    
    ClearCmd                                    ' clear last command
    SendCmd Read_Receiver_Status                ' request S-meter reading
    
    Receive 1                                   ' read response
    If UBound(RxData) = 0 Then                  ' if we got something back
        Reading = RxData(0) Mod 32              ' extract S-meter part
        Form1.pbarSmeter.Value = Reading        ' show the reading
    End If

End Sub

Public Sub ReadFreq()
'***********************************************************
' Read frequency and mode procedure - result returned should be
' 4 bytes of frequency and 1 byte of mode information
'***********************************************************

    ' request data
    ClearCmd                                    ' clear last command
    SendCmd Read_Freq_Mode_Main                 ' request the frequency
    
    ' look for response
    Receive 5                                   ' read response
    
    If UBound(RxData) = 4 Then                  ' make sure got some data
        ' decode the result
        Frequency = Format(Hex(RxData(0)), "00")
        Frequency = Frequency + Format(Hex(RxData(1)), "00")
        Frequency = Frequency + Format(Hex(RxData(2)), "00")
        Frequency = Frequency + Format(Hex(RxData(3)), "00")
        FreqLong = Val(Frequency)               ' save frequency in long format
        
        ' format for display
        ShowFrequency = Left(Frequency, 3) + "." + Mid(Frequency, 4, 3)
        If DecimalsOn Then
            ShowFrequency = ShowFrequency + "." + Right(Frequency, 2)
        End If
        
        ' clear out leading zeros
        For I = 1 To 3
            If Mid(ShowFrequency, I, 1) = "0" Then
                Mid(ShowFrequency, I, 1) = " "
            End If
            If Mid(ShowFrequency, I + 1, 1) <> "0" Then
                Exit For
            End If
        Next
        
        ' show on screem
        Form1.txtFreq.Text = Trim(ShowFrequency)
        
        ' decode mode
        Select Case RxData(4)                   ' final byte is the mode
            Case 0
                Mode = "LSB"
            Case 1
                Mode = "USB"
            Case 2
                Mode = "CW"
            Case 3
                Mode = "CW-R"
            Case 4
                Mode = "AM"
            Case 8
                Mode = "FM"
            Case 82
                Mode = "CW(N)"
            Case 83
                Mode = "CW(N)-R"
            Case 84
                Mode = "AM(N)"
            Case 88
                Mode = "FM(N)"
            Case Else
                Mode = "???"                    ' unknown mode
        End Select
    
        ' show mode
        Form1.txtMode.Text = Trim(Mode)

    End If

End Sub

Public Sub ClearCmd()
'***********************************************************
' this procedure clears the first 4 bytes of the command
'***********************************************************
    
    For I = 0 To 3
        Cmd(I) = 0
    Next

End Sub

Public Sub SendCmd(Command As Byte)
'***********************************************************
' this is a generalised procedure to build a 5 byte command
' string and send it to the serial port.
' Usage - SendCmd <command constant>
'***********************************************************
    
    ' clear the input buffer
    Form1.MSComm1.InBufferCount = 0
        
    ' if com port open -
    If Form1.MSComm1.PortOpen = True Then
        ' get the command
        Cmd(4) = Command
    
        ' Send command to transceiver
        Form1.MSComm1.Output = Cmd()            ' send the command
    
        ' Wait for all the data to be sent.
        Do
            Ret = DoEvents()
        Loop Until Form1.MSComm1.OutBufferCount = 0
    End If
    
End Sub

Public Sub Receive(Bytes As Long)
'***********************************************************
' This procedure receives 'n' bytes from the serial port and
' places it into the variant RxData.
' Usage - Receive <n> where <n> is the number of bytes
'***********************************************************

    ' if com port open -
    If Form1.MSComm1.PortOpen = True Then
        Form1.MSComm1.InputLen = Bytes          ' set no. of bytes
        Do
            Ret = DoEvents()
        Loop Until Form1.MSComm1.InBufferCount = Bytes Or _
                   SettingsRequested            ' wait for them or a SettingsRequested click
                   
                   ' the SettingsRequested check is to give the system a 'kick' if
                   ' we have got out of sync with commands sent and responses received.
                   ' A click on the Settings button will take the program out of a
                   ' receive loop
                   
        RxData = Form1.MSComm1.Input            ' read <n> bytes
    End If
    
End Sub

